/*
 * Copyright (c) 2012 Marsha Chechik, Alessio Di Sandro, Michalis Famelis,
 * Rick Salay, Vivien Suen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    Wentao He - Implementation.
 */
package edu.toronto.cs.se.modelepedia.erdiagram.operator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.toronto.cs.se.modelepedia.erdiagram.Attribute;
import edu.toronto.cs.se.modelepedia.erdiagram.Cardinality;
import edu.toronto.cs.se.modelepedia.erdiagram.ERDiagram;
import edu.toronto.cs.se.modelepedia.erdiagram.Entity;
import edu.toronto.cs.se.modelepedia.erdiagram.ErdiagramFactory;
import edu.toronto.cs.se.modelepedia.erdiagram.Relationship;
import edu.toronto.cs.se.modelepedia.erdiagram.operator.util.ERDiagramMergeUtil;

/**
 *	Generate test data for unit testing.
 */
public class ERDiagramTestDataGenerator {
	
	public static ERDiagram prepareLeftERDiagram() {
		ErdiagramFactory erdiagramFactory = ErdiagramFactory.eINSTANCE;
		ERDiagram erDiagram = erdiagramFactory.createERDiagram();
		
		//prepare entities and their attributes
		Entity entity1 = erdiagramFactory.createEntity();
		entity1.setName("Employee");
		Attribute attribute11 = erdiagramFactory.createAttribute(); 
		Attribute attribute12 = erdiagramFactory.createAttribute();
		Attribute attribute121 = erdiagramFactory.createAttribute();
		Attribute attribute122 = erdiagramFactory.createAttribute();
		Attribute attribute13 = erdiagramFactory.createAttribute();
		Attribute attribute14 = erdiagramFactory.createAttribute();
		attribute11.setName("id");
		attribute11.setKey(true);
		attribute12.setName("name");
		attribute12.setComposite(true);
		attribute121.setName("first_name");
		attribute122.setName("last_name");
		attribute12.getAttributeAttributes().add(attribute121);
		attribute12.getAttributeAttributes().add(attribute122);
		attribute13.setName("date_of_birth");
		attribute14.setName("age");
		attribute14.setDerived(true);
		entity1.getEntityAttributes().add(attribute11);
		entity1.getEntityAttributes().add(attribute12);
		entity1.getEntityAttributes().add(attribute13);
		entity1.getEntityAttributes().add(attribute14);
		
		Entity entity2 = erdiagramFactory.createEntity();
		entity2.setName("Unit");
		Attribute attribute21 = erdiagramFactory.createAttribute();
		Attribute attribute22 = erdiagramFactory.createAttribute();
		attribute21.setName("name");
		attribute21.setKey(true);
		attribute22.setName("head_count");
		entity2.getEntityAttributes().add(attribute21);
		entity2.getEntityAttributes().add(attribute22);
		
		Entity entity3 = erdiagramFactory.createEntity();
		entity3.setName("Project");
		Attribute attribute31 = erdiagramFactory.createAttribute();
		attribute31.setName("title");
		attribute31.setKey(true);
		entity3.getEntityAttributes().add(attribute31);
		
		Entity entity4 = erdiagramFactory.createEntity();
		entity4.setName("Dependent");
		entity4.setWeak(true);
		Attribute attribute41 = erdiagramFactory.createAttribute();
		Attribute attribute42 = erdiagramFactory.createAttribute();
		Attribute attribute43 = erdiagramFactory.createAttribute();
		attribute41.setName("name");
		attribute41.setKey(true);
		attribute42.setName("date_of_birth");
		attribute43.setName("relationship");
		entity4.getEntityAttributes().add(attribute41);
		entity4.getEntityAttributes().add(attribute42);
		entity4.getEntityAttributes().add(attribute43);
		
		//prepare relationships and their attributes
		Relationship relationship1 = erdiagramFactory.createRelationship();
		relationship1.setName("belongs_to");
		relationship1.setFrom(entity1);
		relationship1.setTo(entity2);
		relationship1.setCardinality(Cardinality.MANY_TO_ONE);
		Attribute attribute51 = erdiagramFactory.createAttribute();
		attribute51.setName("start_date");
		relationship1.getRelationshipAttributes().add(attribute51);
		
		Relationship relationship2 = erdiagramFactory.createRelationship();
		relationship2.setName("has");
		relationship2.setFrom(entity2);
		relationship2.setTo(entity3);
		relationship2.setCardinality(Cardinality.ONE_TO_MANY);
		
		Relationship relationship3 = erdiagramFactory.createRelationship();
		relationship3.setName("works_on");
		relationship3.setFrom(entity1);
		relationship3.setTo(entity3);
		relationship3.setCardinality(Cardinality.MANY_TO_ONE);
		
		Relationship relationship4 = erdiagramFactory.createRelationship();
		relationship4.setName("dependent_of");
		relationship4.setWeak(true);
		relationship4.setFrom(entity1);
		relationship4.setTo(entity4);
		relationship4.setCardinality(Cardinality.ONE_TO_MANY);
		
		//add the entities and relationships to the erDiagram
		List<Entity> entityList = new ArrayList<Entity>();
		entityList.add(entity1);
		entityList.add(entity2);
		entityList.add(entity3);
		entityList.add(entity4);
		
		List<Relationship> relationshipList = new ArrayList<Relationship>();
		relationshipList.add(relationship1);
		relationshipList.add(relationship2);
		relationshipList.add(relationship3);
		relationshipList.add(relationship4);
		
		erDiagram.getEntities().addAll(entityList);
		erDiagram.getRelationships().addAll(relationshipList);
		
		return erDiagram;
	}
	
	
	public static ERDiagram prepareRightERDiagram() {
		ErdiagramFactory erdiagramFactory = ErdiagramFactory.eINSTANCE;
		ERDiagram erDiagram = erdiagramFactory.createERDiagram();
		
		//prepare entities and their attributes
		Entity entity1 = erdiagramFactory.createEntity();
		entity1.setName("Employee");
		Attribute attribute11 = erdiagramFactory.createAttribute();
		attribute11.setName("emp_id");
		attribute11.setKey(true);
		entity1.getEntityAttributes().add(attribute11);
		
		Entity entity2 = erdiagramFactory.createEntity();
		entity2.setName("Department");
		Attribute attribute21 = erdiagramFactory.createAttribute();
		Attribute attribute22 = erdiagramFactory.createAttribute();
		Attribute attribute23 = erdiagramFactory.createAttribute();
		attribute21.setName("name");
		attribute21.setKey(true);
		attribute22.setName("employee_count");
		attribute23.setName("locations");
		attribute23.setMultivalued(true);
		entity2.getEntityAttributes().add(attribute21);
		entity2.getEntityAttributes().add(attribute22);
		entity2.getEntityAttributes().add(attribute23);
		
		Entity entity3 = erdiagramFactory.createEntity();
		entity3.setName("Project");
		Attribute attribute31 = erdiagramFactory.createAttribute();
		Attribute attribute32 = erdiagramFactory.createAttribute();
		Attribute attribute33 = erdiagramFactory.createAttribute();
		attribute31.setName("name");
		attribute31.setKey(true);
		attribute32.setName("start_date");
		attribute33.setName("size");
		entity3.getEntityAttributes().add(attribute31);
		entity3.getEntityAttributes().add(attribute32);
		entity3.getEntityAttributes().add(attribute33);
		
		Entity entity4 = erdiagramFactory.createEntity();
		entity4.setName("Module");
		entity4.setWeak(true);
		Attribute attribute41 = erdiagramFactory.createAttribute();
		Attribute attribute42 = erdiagramFactory.createAttribute();
		attribute41.setName("name");
		attribute41.setKey(true);
		attribute42.setName("start_date");
		entity4.getEntityAttributes().add(attribute41);
		entity4.getEntityAttributes().add(attribute42);
		
		//prepare relationships and their attributes
		Relationship relationship1 = erdiagramFactory.createRelationship();
		relationship1.setName("manages");
		relationship1.setFrom(entity1);
		relationship1.setTo(entity2);
		relationship1.setCardinality(Cardinality.ONE_TO_ONE);
		Attribute attribute51 = erdiagramFactory.createAttribute();
		attribute51.setName("start_date");
		relationship1.getRelationshipAttributes().add(attribute51);
		
		Relationship relationship2 = erdiagramFactory.createRelationship();
		relationship2.setName("works_for");
		relationship2.setFrom(entity1);
		relationship2.setTo(entity2);
		relationship2.setCardinality(Cardinality.MANY_TO_ONE);
		Attribute attribute61 = erdiagramFactory.createAttribute();
		attribute61.setName("since");
		relationship2.getRelationshipAttributes().add(attribute61);
		
		Relationship relationship3 = erdiagramFactory.createRelationship();
		relationship3.setName("supervises");
		relationship3.setFrom(entity1);
		relationship3.setTo(entity1);
		relationship3.setCardinality(Cardinality.ONE_TO_MANY);
		
		Relationship relationship4 = erdiagramFactory.createRelationship();
		relationship4.setName("owns");
		relationship4.setFrom(entity2);
		relationship4.setTo(entity3);
		relationship4.setCardinality(Cardinality.ONE_TO_MANY);
		
		Relationship relationship5 = erdiagramFactory.createRelationship();
		relationship5.setName("contains");
		relationship5.setWeak(true);
		relationship5.setFrom(entity3);
		relationship5.setTo(entity4);
		relationship5.setCardinality(Cardinality.ONE_TO_MANY);
		
		//add the entities and relationships to the erDiagram
		List<Entity> entityList = new ArrayList<Entity>();
		entityList.add(entity1);
		entityList.add(entity2);
		entityList.add(entity3);
		entityList.add(entity4);
		
		List<Relationship> relationshipList = new ArrayList<Relationship>();
		relationshipList.add(relationship1);
		relationshipList.add(relationship2);
		relationshipList.add(relationship3);
		relationshipList.add(relationship4);
		relationshipList.add(relationship5);
		
		erDiagram.getEntities().addAll(entityList);
		erDiagram.getRelationships().addAll(relationshipList);
		
		return erDiagram;
	}
	
	
	public static ERDiagram prepareConnector() {
		ErdiagramFactory erdiagramFactory = ErdiagramFactory.eINSTANCE;
		ERDiagram erDiagram = erdiagramFactory.createERDiagram();
		
		//prepare entities and their attributes
		Entity entity1 = erdiagramFactory.createEntity();
		entity1.setName("Employee");
		Attribute attribute11 = erdiagramFactory.createAttribute();
		attribute11.setName("emp_id");
		attribute11.setKey(true);
		entity1.getEntityAttributes().add(attribute11);
		
		Entity entity2 = erdiagramFactory.createEntity();
		entity2.setName("Department");
		Attribute attribute21 = erdiagramFactory.createAttribute();
		Attribute attribute22 = erdiagramFactory.createAttribute();
		attribute21.setName("name");
		attribute21.setKey(true);
		attribute22.setName("employee_count");
		entity2.getEntityAttributes().add(attribute21);
		entity2.getEntityAttributes().add(attribute22);
		
		Entity entity3 = erdiagramFactory.createEntity();
		entity3.setName("Project");
		Attribute attribute31 = erdiagramFactory.createAttribute();
		attribute31.setName("name");
		attribute31.setKey(true);
		entity3.getEntityAttributes().add(attribute31);
			
		//prepare relationships and their attributes
		Relationship relationship1 = erdiagramFactory.createRelationship();
		relationship1.setName("works_for");
		relationship1.setFrom(entity1);
		relationship1.setTo(entity2);
		relationship1.setCardinality(Cardinality.MANY_TO_ONE);
		Attribute attribute41 = erdiagramFactory.createAttribute();
		attribute41.setName("since");
		relationship1.getRelationshipAttributes().add(attribute41);
		
		Relationship relationship2 = erdiagramFactory.createRelationship();
		relationship2.setName("owns");
		relationship2.setFrom(entity2);
		relationship2.setTo(entity3);
		relationship2.setCardinality(Cardinality.ONE_TO_MANY);
				
		//add the entities and relationships to the erDiagram
		List<Entity> entityList = new ArrayList<Entity>();
		entityList.add(entity1);
		entityList.add(entity2);
		entityList.add(entity3);
		
		List<Relationship> relationshipList = new ArrayList<Relationship>();
		relationshipList.add(relationship1);
		relationshipList.add(relationship2);
		
		erDiagram.getEntities().addAll(entityList);
		erDiagram.getRelationships().addAll(relationshipList);
		
		return erDiagram;		
	}
	
	
	public static Map<String, String> prepareLeftToRightDiagramMapping() {
		Map<String, String> leftToRightDiagramMapping = new HashMap<String, String>();
		//prepare entity mapping
		leftToRightDiagramMapping.put("ENTITY@Employee", "ENTITY@Employee");		
		leftToRightDiagramMapping.put("ENTITY@Unit", "ENTITY@Department");
		leftToRightDiagramMapping.put("ENTITY@Project", "ENTITY@Project");
		//prepare relationship mapping
		leftToRightDiagramMapping.put("RELATIONSHIP@Employee-belongs_to-Unit", "RELATIONSHIP@Employee-works_for-Department");
		leftToRightDiagramMapping.put("RELATIONSHIP@Unit-has-Project", "RELATIONSHIP@Department-owns-Project");
		//prepare entity attribute mapping
		leftToRightDiagramMapping.put("ENTITY_ATTRIBUTE@Employee-id", "ENTITY_ATTRIBUTE@Employee-emp_id");
		leftToRightDiagramMapping.put("ENTITY_ATTRIBUTE@Unit-name", "ENTITY_ATTRIBUTE@Department-name");
		leftToRightDiagramMapping.put("ENTITY_ATTRIBUTE@Unit-head_count", "ENTITY_ATTRIBUTE@Department-employee_count");
		leftToRightDiagramMapping.put("ENTITY_ATTRIBUTE@Project-title", "ENTITY_ATTRIBUTE@Project-name");
		//prepare relationship attribute mapping
		leftToRightDiagramMapping.put("RELATIONSHIP_ATTRIBUTE@Employee-belongs_to-Unit-start_date", 
				"RELATIONSHIP_ATTRIBUTE@Employee-works_for-Department-since");
		
		return leftToRightDiagramMapping;
	}
	

	public static Map<String, String> prepareConnectorToLeftDiagramEntityMapping() {
		Map<String, String> entityMapping = new HashMap<String, String>();
		entityMapping.put("ENTITY@Employee", "ENTITY@Employee");
		entityMapping.put("ENTITY@Department", "ENTITY@Unit");
		entityMapping.put("ENTITY@Project", "ENTITY@Project");
		return entityMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToRightDiagramEntityMapping() {
		Map<String, String> entityMapping = new HashMap<String, String>();
		entityMapping.put("ENTITY@Employee", "ENTITY@Employee");
		entityMapping.put("ENTITY@Department", "ENTITY@Department");
		entityMapping.put("ENTITY@Project", "ENTITY@Project");
		return entityMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToLeftDiagramRelationshipMapping() {
		Map<String, String> relationshipMapping = new HashMap<String, String>();
		relationshipMapping.put("RELATIONSHIP@Employee-works_for-Department", "RELATIONSHIP@Employee-belongs_to-Unit");
		relationshipMapping.put("RELATIONSHIP@Department-owns-Project", "RELATIONSHIP@Unit-has-Project");
		return relationshipMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToRightDiagramRelationshipMapping() {
		Map<String, String> relationshipMapping = new HashMap<String, String>();
		relationshipMapping.put("RELATIONSHIP@Employee-works_for-Department", "RELATIONSHIP@Employee-works_for-Department");
		relationshipMapping.put("RELATIONSHIP@Department-owns-Project", "RELATIONSHIP@Department-owns-Project");
		return relationshipMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToLeftDiagramEntityAttributeMapping() {
		Map<String, String> entityAttributeMapping = new HashMap<String, String>();
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Employee-emp_id", "ENTITY_ATTRIBUTE@Employee-id");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Department-name", "ENTITY_ATTRIBUTE@Unit-name");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Department-employee_count", "ENTITY_ATTRIBUTE@Unit-head_count");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Project-name", "ENTITY_ATTRIBUTE@Project-title");
		return entityAttributeMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToRightDiagramEntityAttributeMapping() {
		Map<String, String> entityAttributeMapping = new HashMap<String, String>();
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Employee-emp_id", "ENTITY_ATTRIBUTE@Employee-emp_id");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Department-name", "ENTITY_ATTRIBUTE@Department-name");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Department-employee_count", "ENTITY_ATTRIBUTE@Department-employee_count");
		entityAttributeMapping.put("ENTITY_ATTRIBUTE@Project-name", "ENTITY_ATTRIBUTE@Project-name");
		return entityAttributeMapping;
	}
	

	public static Map<String, String> prepareConnectorToLeftDiagramRelationshipAttributeMapping() {
		Map<String, String> relationshipAttributeMapping = new HashMap<String, String>();
		relationshipAttributeMapping.put("RELATIONSHIP_ATTRIBUTE@Employee-works_for-Department-since", 
				"RELATIONSHIP_ATTRIBUTE@Employee-belongs_to-Unit-start_date");
		return relationshipAttributeMapping;
	}
	
	
	public static Map<String, String> prepareConnectorToRightDiagramRelationshipAttributeMapping() {
		Map<String, String> relationshipAttributeMapping = new HashMap<String, String>();
		relationshipAttributeMapping.put("RELATIONSHIP_ATTRIBUTE@Employee-works_for-Department-since", 
				"RELATIONSHIP_ATTRIBUTE@Employee-works_for-Department-since");
		return relationshipAttributeMapping;
	}
	

	public static List<String> prepareEntityDisjointUnion() {
		List<String> entityDisjointUnion = new ArrayList<String>();
		entityDisjointUnion.add("Employee@Left");
		entityDisjointUnion.add("Unit@Left");
		entityDisjointUnion.add("Project@Left");
		entityDisjointUnion.add("Dependent@Left");
		entityDisjointUnion.add("Employee@Connector");
		entityDisjointUnion.add("Department@Connector");
		entityDisjointUnion.add("Project@Connector");
		entityDisjointUnion.add("Employee@Right");
		entityDisjointUnion.add("Department@Right");
		entityDisjointUnion.add("Project@Right");
		entityDisjointUnion.add("Module@Right");
		return entityDisjointUnion;
	}
	
	
	public static List<String> prepareRelationshipDisjointUnion() {
		List<String> relationshipDisjointUnion = new ArrayList<String>();
		relationshipDisjointUnion.add("Employee-belongs_to-Unit@Left");
		relationshipDisjointUnion.add("Employee-works_on-Project@Left");
		relationshipDisjointUnion.add("Unit-has-Project@Left");
		relationshipDisjointUnion.add("Employee-dependent_of-Dependent@Left");
		relationshipDisjointUnion.add("Employee-works_for-Department@Connector");
		relationshipDisjointUnion.add("Department-owns-Project@Connector");
		relationshipDisjointUnion.add("Employee-manages-Department@Right");
		relationshipDisjointUnion.add("Employee-works_for-Department@Right");
		relationshipDisjointUnion.add("Employee-supervises-Employee@Right");
		relationshipDisjointUnion.add("Department-owns-Project@Right");
		relationshipDisjointUnion.add("Project-contains-Module@Right");
		return relationshipDisjointUnion;
	}
	
	
	public static List<String> prepareEntityAttributeDisjointUnion() {
		List<String> entityAttributeDisjointUnion = new ArrayList<String>();
		entityAttributeDisjointUnion.add("Employee-id@Left");
		entityAttributeDisjointUnion.add("Employee-date_of_birth@Left");
		entityAttributeDisjointUnion.add("Employee-age@Left");
		entityAttributeDisjointUnion.add("Employee-name@Left");
		entityAttributeDisjointUnion.add("Employee-name^last_name@Left");
		entityAttributeDisjointUnion.add("Employee-name^first_name@Left");
		entityAttributeDisjointUnion.add("Unit-name@Left");
		entityAttributeDisjointUnion.add("Unit-head_count@Left");
		entityAttributeDisjointUnion.add("Project-title@Left");
		entityAttributeDisjointUnion.add("Dependent-name@Left");
		entityAttributeDisjointUnion.add("Dependent-relationship@Left");
		entityAttributeDisjointUnion.add("Dependent-date_of_birth@Left");
		entityAttributeDisjointUnion.add("Employee-emp_id@Connector");
		entityAttributeDisjointUnion.add("Department-name@Connector");
		entityAttributeDisjointUnion.add("Department-employee_count@Connector");
		entityAttributeDisjointUnion.add("Project-name@Connector");
		entityAttributeDisjointUnion.add("Employee-emp_id@Right");
		entityAttributeDisjointUnion.add("Department-name@Right");
		entityAttributeDisjointUnion.add("Department-locations@Right");
		entityAttributeDisjointUnion.add("Department-employee_count@Right");
		entityAttributeDisjointUnion.add("Project-size@Right");
		entityAttributeDisjointUnion.add("Project-name@Right");
		entityAttributeDisjointUnion.add("Project-start_date@Right");
		entityAttributeDisjointUnion.add("Module-name@Right");
		entityAttributeDisjointUnion.add("Module-start_date@Right");
		return entityAttributeDisjointUnion;
	}
	
	
	public static List<String> prepareRelationshipAttributeDisjointUnion() {
		List<String> relationshipAttributeDisjointUnion = new ArrayList<String>();
		relationshipAttributeDisjointUnion.add("Employee-belongs_to-Unit-start_date@Left");
		relationshipAttributeDisjointUnion.add("Employee-works_for-Department-since@Connector");
		relationshipAttributeDisjointUnion.add("Employee-manages-Department-start_date@Right");
		relationshipAttributeDisjointUnion.add("Employee-works_for-Department-since@Right");
		return relationshipAttributeDisjointUnion;
	}
	
	
	public static ERDiagram prepareERDiagram(int count, String namePrefix) {
		ErdiagramFactory erdiagramFactory = ErdiagramFactory.eINSTANCE;
		ERDiagram erdiagram = erdiagramFactory.createERDiagram();
		
		//prepare entities and their attributes
		for(int i=0; i<count; i++) {
			Entity entity = erdiagramFactory.createEntity();
			entity.setName(namePrefix+"Entity"+Integer.toString(i));
			Attribute attribute = erdiagramFactory.createAttribute();
			attribute.setName(namePrefix+"Attribute"+Integer.toString(i));
			entity.getEntityAttributes().add(attribute);
			erdiagram.getEntities().add(entity);
		}
		//prepare relationships and their attributes
		for(int j=0; j<count; j++) {
			Relationship relationship = erdiagramFactory.createRelationship();
			relationship.setName(namePrefix+"Relationship"+Integer.toString(j));
			Attribute attribute = erdiagramFactory.createAttribute();
			attribute.setName(namePrefix+"Attribute"+Integer.toString(j));
			relationship.getRelationshipAttributes().add(attribute);
			relationship.setFrom(ERDiagramMergeUtil.lookupEntity(erdiagram, namePrefix+"Entity"+Integer.toString(j)));
			relationship.setTo(ERDiagramMergeUtil.lookupEntity(erdiagram, namePrefix+"Entity"+Integer.toString(j+1>=count?0:j+1)));
			erdiagram.getRelationships().add(relationship);
		}
		
		return erdiagram;
	}
	
	
	public static Map<String, String> prepareLeftToRightDiagramMapping(int count) {
		Map<String, String> leftToRightDiagramMapping = new HashMap<String, String>();
		
		for(int i=0; i<count; i++) {
			//prepare entity mapping
			leftToRightDiagramMapping.put("ENTITY@leftEntity"+i, "ENTITY@rightEntity"+i);		
			//prepare relationship mapping
			leftToRightDiagramMapping.put("RELATIONSHIP@leftEntity"+i+"-leftRelationship"+i+"-leftEntity"+(i+1>=count?0:i+1), 
					"RELATIONSHIP@rightEntity"+i+"-rightRelationship"+i+"-rightEntity"+(i+1>=count?0:i+1));
			//prepare entity attribute mapping
			leftToRightDiagramMapping.put("ENTITY_ATTRIBUTE@leftEntity"+i+"-"+"leftAttribute"+i, 
					"ENTITY_ATTRIBUTE@rightEntity"+i+"-"+"rightAttribute"+i);
			//prepare relationship attribute mapping
			leftToRightDiagramMapping.put("RELATIONSHIP_ATTRIBUTE@leftEntity"+i+"-leftRelationship"+i+"-leftEntity"+(i+1>=count?0:i+1)+"-leftAttribute"+i, 
					"RELATIONSHIP_ATTRIBUTE@rightEntity"+i+"-rightRelationship"+i+"-rightEntity"+(i+1>=count?0:i+1)+"-rightAttribute"+i);
		}

		return leftToRightDiagramMapping;
	}
	
}
