/*
 * Copyright (c) 2012 Marsha Chechik, Alessio Di Sandro, Michalis Famelis,
 * Rick Salay, Vivien Suen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    Wentao He - Implementation.
 */
package edu.toronto.cs.se.modelepedia.erdiagram.operator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import edu.toronto.cs.se.modelepedia.erdiagram.ERDiagram;

import junit.framework.TestCase;


/**
 * Test the performance of the ER diagram merge operator. The elapsed time will be printed out.
 * 
 * How to use: 
 * The following three global variables can be adjusted - initCount, increases and nunberOfRounds.  
 * 		initCount is the number of entity/relationship/attribute in each input ER diagram for the 1st round of run;
 * 		increases is the number of entity/relationship/attribute to be added to the input ER diagrams in the previous run.
 * 		numberOfRounds is the number of rounds this test case will be automatically executed.
 */
public class TestERDiagramMergePerformance extends TestCase {
	
	int initCount = 50;
	int increases = 50;
	int numberOfRounds = 20;
	
	ERDiagramMerge erdiagramMerge;
	ERDiagram leftERDiagram;
	ERDiagram rightERDiagram;
	Map<String, String> leftToRightDiagramMapping; 
	List<Integer> scaleList;
	
	private static final String METHOD_MERGE = "merge";
	
	protected void setUp() throws Exception {		
		erdiagramMerge = new ERDiagramMerge();
		scaleList = new ArrayList<Integer>();
	}
	
	
	public void testMerge() {
		try {
			Method method = erdiagramMerge.getClass().getDeclaredMethod(METHOD_MERGE, new Class[]{ERDiagram.class, ERDiagram.class, Map.class});
			method.setAccessible(true); 
			
			int count = initCount;
			for(int i=0; i<numberOfRounds; i++) {
				leftERDiagram = ERDiagramTestDataGenerator.prepareERDiagram(count, "left");
				rightERDiagram = ERDiagramTestDataGenerator.prepareERDiagram(count, "right");
				leftToRightDiagramMapping = ERDiagramTestDataGenerator.prepareLeftToRightDiagramMapping(count); 
				
				long startTime = System.currentTimeMillis();
				method.invoke(erdiagramMerge, new Object[]{leftERDiagram, rightERDiagram, leftToRightDiagramMapping});
				long stopTime = System.currentTimeMillis();
			    long elapsedTime = stopTime - startTime;
				
			    System.out.println("Elapsed " + elapsedTime + " milli seconds for scale " + count);
			    
			    count = count + increases;
			}
			method.setAccessible(false);
			
		} catch (IllegalArgumentException e) {
		    e.printStackTrace();
		   } catch (NoSuchMethodException e) {
		    e.printStackTrace();
		   } catch (IllegalAccessException e) {
		    e.printStackTrace();
		   } catch (InvocationTargetException e) {
		    e.printStackTrace();
		   }			
	}
	
}
